@@ -2,8 +2,10 @@ import warnings
 import numpy as np
 import pandas as pd
 from sklearn.base import BaseEstimator
-from sklearn.utils.validation import check_consistent_length
 from sklearn.utils.multiclass import type_of_target
+from sklearn.utils.validation import check_consistent_length
+
+from ..utils import check_is_binary
 
 
 class SoloModel(BaseEstimator):
@@ -11,16 +13,21 @@ class SoloModel(BaseEstimator):
 
     Fit solo model on whole dataset with 'treatment' as an additional feature.
 
-    For each test example calculate predictions on new set twice:
-    with treatment == '1' and with treatment == '0'.
-    After that calculate uplift as a delta between these predictions.
+    Each object from the test sample is scored twice: with the communication flag equal to 1 and equal to 0.
+    Subtracting the probabilities for each observation, we get the uplift.
 
     Return delta of predictions for each example.
 
-    See more details about `SoloModel in documentation`_.
+    Read more in the :ref:`User Guide <SoloModel>`.
 
     Args:
         estimator (estimator object implementing 'fit'): The object to use to fit the data.
+        method (string, ’dummy’ or ’treatment_interaction’, default='dummy'): Specifies the approach:
+        
+            * ``'dummy'``:
+                Single model;
+            * ``'treatment_interaction'``:
+                Single model including treatment interactions.
 
     Attributes:
         trmnt_preds_ (array-like, shape (n_samples, )): Estimator predictions on samples when treatment.
@@ -42,16 +49,30 @@ class SoloModel(BaseEstimator):
         Lo, Victor. (2002). The True Lift Model - A Novel Data Mining Approach to Response Modeling
         in Database Marketing. SIGKDD Explorations. 4. 78-86.
 
-    .. _SoloModel in documentation:
-        https://scikit-uplift.readthedocs.io/en/latest/api/models.html#one-model-with-treatment-as-feature
+    See Also:
+
+        **Other approaches:**
+
+        * :class:`.ClassTransformation`: Class Variable Transformation approach.
+        * :class:`.TwoModels`: Double classifier approach.
+
+        **Other:**
+
+        * :func:`.plot_uplift_preds`: Plot histograms of treatment, control and uplift predictions.
     """
 
-    def __init__(self, estimator):
+    def __init__(self, estimator, method='dummy'):
         self.estimator = estimator
+        self.method = method
         self.trmnt_preds_ = None
         self.ctrl_preds_ = None
         self._type_of_target = None
 
+        all_methods = ['dummy', 'treatment_interaction']
+        if method not in all_methods:
+            raise ValueError("SoloModel approach supports only methods in %s, got"
+                             " %s." % (all_methods, method))
+
     def fit(self, X, y, treatment, estimator_fit_params=None):
         """Fit the model according to the given training data.
 
@@ -72,16 +93,32 @@ class SoloModel(BaseEstimator):
         """
 
         check_consistent_length(X, y, treatment)
+        check_is_binary(treatment)
         treatment_values = np.unique(treatment)
+
         if len(treatment_values) != 2:
             raise ValueError("Expected only two unique values, got %s" % len(treatment_values))
 
-        if isinstance(X, np.ndarray):
-            X_mod = np.column_stack((X, treatment))
-        elif isinstance(X, pd.DataFrame):
-            X_mod = X.assign(treatment=treatment)
-        else:
-            raise TypeError("Expected numpy.ndarray or pandas.DataFrame in training vector X, got %s" % type(X))
+        if self.method == 'dummy':
+            if isinstance(X, np.ndarray):
+                X_mod = np.column_stack((X, treatment))
+            elif isinstance(X, pd.DataFrame):
+                X_mod = X.assign(treatment=treatment)
+            else:
+                raise TypeError("Expected numpy.ndarray or pandas.DataFrame in training vector X, got %s" % type(X))
+
+        if self.method == 'treatment_interaction':
+            if isinstance(X, np.ndarray):
+                X_mod = np.column_stack((X, np.multiply(X, np.array(treatment).reshape(-1, 1)), treatment))
+            elif isinstance(X, pd.DataFrame):
+                X_mod = pd.concat([
+                    X,
+                    X.apply(lambda x: x * treatment)
+                        .rename(columns=lambda x: str(x) + '_treatment_interaction')
+                ], axis=1) \
+                    .assign(treatment=treatment)
+            else:
+                raise TypeError("Expected numpy.ndarray or pandas.DataFrame in training vector X, got %s" % type(X))
 
         self._type_of_target = type_of_target(y)
 
@@ -100,14 +137,36 @@ class SoloModel(BaseEstimator):
         Returns:
             array (shape (n_samples,)): uplift
         """
-        if isinstance(X, np.ndarray):
-            X_mod_trmnt = np.column_stack((X, np.ones(X.shape[0])))
-            X_mod_ctrl = np.column_stack((X, np.zeros(X.shape[0])))
-        elif isinstance(X, pd.DataFrame):
-            X_mod_trmnt = X.assign(treatment=np.ones(X.shape[0]))
-            X_mod_ctrl = X.assign(treatment=np.zeros(X.shape[0]))
-        else:
-            raise TypeError("Expected numpy.ndarray or pandas.DataFrame in training vector X, got %s" % type(X))
+
+        if self.method == 'dummy':
+            if isinstance(X, np.ndarray):
+                X_mod_trmnt = np.column_stack((X, np.ones(X.shape[0])))
+                X_mod_ctrl = np.column_stack((X, np.zeros(X.shape[0])))
+            elif isinstance(X, pd.DataFrame):
+                X_mod_trmnt = X.assign(treatment=np.ones(X.shape[0]))
+                X_mod_ctrl = X.assign(treatment=np.zeros(X.shape[0]))
+            else:
+                raise TypeError("Expected numpy.ndarray or pandas.DataFrame in training vector X, got %s" % type(X))
+
+        if self.method == 'treatment_interaction':
+            if isinstance(X, np.ndarray):
+                X_mod_trmnt = np.column_stack((X, np.multiply(X, np.ones((X.shape[0], 1))), np.ones(X.shape[0])))
+                X_mod_ctrl = np.column_stack((X, np.multiply(X, np.zeros((X.shape[0], 1))), np.zeros(X.shape[0])))
+            elif isinstance(X, pd.DataFrame):
+                X_mod_trmnt = pd.concat([
+                    X,
+                    X.apply(lambda x: x * np.ones(X.shape[0]))
+                        .rename(columns=lambda x: str(x) + '_treatment_interaction')
+                ], axis=1) \
+                    .assign(treatment=np.ones(X.shape[0]))
+                X_mod_ctrl = pd.concat([
+                    X,
+                    X.apply(lambda x: x * np.zeros(X.shape[0]))
+                        .rename(columns=lambda x: str(x) + '_treatment_interaction')
+                ], axis=1) \
+                    .assign(treatment=np.zeros(X.shape[0]))
+            else:
+                raise TypeError("Expected numpy.ndarray or pandas.DataFrame in training vector X, got %s" % type(X))
 
         if self._type_of_target == 'binary':
             self.trmnt_preds_ = self.estimator.predict_proba(X_mod_trmnt)[:, 1]
@@ -124,18 +183,15 @@ class ClassTransformation(BaseEstimator):
     """aka Class Variable Transformation or Revert Label approach.
 
     Redefine target variable, which indicates that treatment make some impact on target or
-    did target is negative without treatment.
+    did target is negative without treatment: ``Z = Y * W + (1 - Y)(1 - W)``,
 
-    Z = Y * W + (1 - Y)(1 - W),
+    where ``Y`` - target vector, ``W`` - vector of binary communication flags.
 
-    where Y - target, W - communication flag.
-
-    Then, Uplift ~ 2 * (Z == 1) - 1
+    Then, ``Uplift ~ 2 * (Z == 1) - 1``
 
     Returns only uplift predictions.
 
-    See more details about `ClassTransformation in documentation`_.
-
+    Read more in the :ref:`User Guide <ClassTransformation>`.
 
     Args:
         estimator (estimator object implementing 'fit'): The object to use to fit the data.
@@ -148,17 +204,25 @@ class ClassTransformation(BaseEstimator):
         from catboost import CatBoostClassifier
 
 
-        ct = ClassTransformation(CatBoostClassifier(verbose=100, random_state=777))  # define approach
-        ct = ct.fit(X_train, y_train, treat_train, estimator_fit_params={{'plot': True})  # fit the model
-        uplift_ct = ct.predict(X_val)  # predict uplift
+        # define approach
+        ct = ClassTransformation(CatBoostClassifier(verbose=100, random_state=777))
+        # fit the model
+        ct = ct.fit(X_train, y_train, treat_train, estimator_fit_params={{'plot': True})
+        # predict uplift
+        uplift_ct = ct.predict(X_val)
 
     References:
         Maciej Jaskowski and Szymon Jaroszewicz. Uplift modeling for clinical trial data.
         ICML Workshop on Clinical Data Analysis, 2012.
 
-    .. _ClassTransformation in documentation:
-        https://scikit-uplift.readthedocs.io/en/latest/api/models.html#class-transformation
+    See Also:
+
+        **Other approaches:**
+
+        * :class:`.SoloModel`: Single model approach.
+        * :class:`.TwoModels`: Double classifier approach.
     """
+
     def __init__(self, estimator):
         self.estimator = estimator
         self._type_of_target = None
@@ -177,8 +241,8 @@ class ClassTransformation(BaseEstimator):
             object: self
         """
 
-        # TODO: check the treatment is binary
         check_consistent_length(X, y, treatment)
+        check_is_binary(treatment)
         self._type_of_target = type_of_target(y)
 
         if self._type_of_target != 'binary':
@@ -217,7 +281,7 @@ class TwoModels(BaseEstimator):
 
     Fit two separate models: on the treatment data and on the control data.
 
-    See more details about `TwoModels in documentation`_.
+    Read more in the :ref:`User Guide <TwoModels>`.
 
     Args:
         estimator_trmnt (estimator object implementing 'fit'): The object to use to fit the treatment data.
@@ -271,8 +335,16 @@ class TwoModels(BaseEstimator):
         Uplift Modeling with Multiple Treatments and General Response Types.
         10.1137/1.9781611974973.66.
 
-    .. _TwoModels in documentation:
-        https://scikit-uplift.readthedocs.io/en/latest/api/models.html#one-model-with-treatment-as-feature
+    See Also:
+
+        **Other approaches:**
+
+        * :class:`.SoloModel`: Single model approach.
+        * :class:`.ClassTransformation`: Class Variable Transformation approach.
+
+        **Other:**
+
+        * :func:`.plot_uplift_preds`: Plot histograms of treatment, control and uplift predictions.
     """
 
     def __init__(self, estimator_trmnt, estimator_ctrl, method='vanilla'):
@@ -312,8 +384,9 @@ class TwoModels(BaseEstimator):
         Returns:
             object: self
         """
-        # TODO: check the treatment is binary
+
         check_consistent_length(X, y, treatment)
+        check_is_binary(treatment)
         self._type_of_target = type_of_target(y)
 
         X_ctrl, y_ctrl = X[treatment == 0], y[treatment == 0]
